library(plyr)
library(ggplot2)
library(stargazer)
library(mfx)
library(miceadds)
source("vcovCluster.r")
library(car)

topic <- "clim" #clim for climate, gun for gun policy, immi for immigration

pol_thres <- .5 #politicalness threshold (certainty needed to classify shows as being political)
lib_thres <- .5 #ideology threshold (certainty needed to classify shows as having ideology)
NPR <- F #include NPR and news shows (in train set and data)
#setting for main analyses are thresholds .5 and NPR False



# Data prep ---------------------------------------------------------------

#get data
d_all <- read.csv(paste0("data/mentions_showweek_", topic, ".csv"))

#binarize political and assign ideology
d_all$political <- d_all$Prob.political >= pol_thres
if(NPR){
  d_all$ideology <- ifelse(d_all$Prob.liberal >= .5, "liberal", "conservative")
} else {
  #if needed, also drop NPR shows and shows with "news" in the name
  d_all$ideology <- ifelse(d_all$NPR | grepl("news", tolower(d_all$show)), NA,
                           ifelse(d_all$Prob.libnoNPR >= lib_thres, "liberal",
                                  ifelse(d_all$Prob.libnoNPR < 1-lib_thres,
                                         "conservative", NA)))
}

#only keep political (and ideological) shows
d <- d_all[d_all$political & !is.na(d_all$political) &
             !is.na(d_all$ideology),]

#pre/post-event week variable as factor
d$prepost <- factor(d$prepost, levels=c("pre","post"))

#data frame for predictions from models
d_pred <- expand.grid(ideology=c("conservative", "liberal"),
                      prepost=c("pre","post"), mins=240)



# Mention counts: Partisan issue selection ----------------------------------------

#neg. binomial model on mention counts
fit_nb <- glm.nb(Total ~ prepost*ideology + mins, data = d)
pred_nb <- cbind(d_pred, count=predict(fit_nb,
                                       d_pred,
                                       type="response"))

print("predicted number of mentions per week on a cons./lib. show pre and post:")
pred_nb

print("hypothesis tests of change for conservative/liberal shows, and difference between them:")
#hypothesis test: change for conservative
linearHypothesis(fit_nb, "prepostpost=0", vcov.=vcovCluster(fit_nb, d$show))
#hypothesis test: change for liberal
linearHypothesis(fit_nb, "prepostpost+prepostpost:ideologyliberal=0", vcov.=vcovCluster(fit_nb, d$show))
#hypothesis test: difference in change between conservative and liberal
linearHypothesis(fit_nb, "prepostpost:ideologyliberal=0", vcov.=vcovCluster(fit_nb, d$show))
#note: 1st and 3rd test can be verified with coeftest(fit_nb, vcov = vcovCluster(fit_nb, d$show))

#robustness check: non-neutral mentions only
d$prop_neither <- d$Neither / (d$Total - d$Not.coded)
d$Total_neither <- ifelse(d$Not.coded==0, d$Neither,
                          round(d$prop_neither*d$Total))
mean(d$Not.coded!=0) #pct observations that were imputed
fit_nonneither <- glm.nb(Total_neither ~ prepost*ideology + mins, data = d)
cbind(d_pred, count=predict(fit_nonneither, d_pred, type="response"))



# Frame proportions: Partisan issue framing ----------------------------

#fractional logit model on frame proportions
fit_fl <- glm.cluster(formula=prop_conc ~ prepost*ideology, data=d,
                          cluster="show", family=quasibinomial('logit'))
fit_fl_noclust <- glm(prop_conc ~ prepost*ideology, d, family=quasibinomial('logit'))
pred_fl <- cbind(d_pred, position=predict(fit_fl_noclust, d_pred, type="response"))

print("predicted proportion of pro-climate/anti-gun/anti-immigr. mentions on a cons./lib. show pre and post:")
pred_fl

print("hypothesis tests of change for conservative/liberal shows, and difference between them:")
#hypothesis test: conservative-liberal difference
linearHypothesis(fit_fl, "ideologyliberal=0")
#hypothesis test: change for conservative
linearHypothesis(fit_fl, "prepostpost=0")
#hypothesis test: change for liberal
linearHypothesis(fit_fl, "prepostpost+prepostpost:ideologyliberal=0")
#note: 1st and 2nd test can be verified with summary(fit_fl)



# Store results for ideology threshold appendix ---------------------------

#combine all predictions
pred <- merge(pred_nb, pred_fl)

if(!NPR){
  #analyses for ideology thresholds
  pred$pol_thres <- pol_thres
  pred$lib_thres <- lib_thres
  write.csv(pred, paste0("supplementary/ideo_npr_app/pred_", topic, "_", pol_thres,
                         "_", lib_thres, ".csv"), row.names=F)
} else if (pol_thres == .5 & lib_thres == .5){
  #analyses with NPR
  write.csv(pred, paste0("supplementary/ideo_npr_app/pred_withNPR_", topic,
                         ".csv"), row.names=F)
}



# Store results for regression tables -------------------------------------

#only if parameters are "standard"; also do John Fredericks robustness check

if(!NPR & lib_thres == .5 & pol_thres == .5){
  
  coef_nb <- coeftest(fit_nb, vcov = vcovCluster(fit_nb, d$show))
  coef_fl <- coeftest(fit_fl) #model already includes cluster option
  save(fit_nb, coef_nb, fit_fl_noclust, coef_fl, file=paste0("regtables/modobj_", topic, ".Rdata"))
  
  #robustness check for counts: leave out The John Fredericks Show, which appears
  #to be recorded (nearly) around the clock on WHKT (many repeated mentions
  #confirm this) and therefore has a very large (outlier) minute count
  d_noJHF <- d[d$show != "The John Fredericks Show",]
  fit_nb_noJHF <- glm.nb(Total ~ prepost*ideology + mins, data = d_noJHF)
  cbind(pred_nb, count_noJHF=predict(fit_nb_noJHF, d_pred, type="response"))
  rbind(fit_nb$coefficients, fit_nb_noJHF$coefficients)
  
}



# Store results for figures with bootstrapped CIs -------------------------

#only if parameters are "standard"

if(!NPR & lib_thres == .5 & pol_thres == .5){
  
  #one clustered bootstrap sample
  one_bs_sample <- function(d){
    
    #extract shows with replacement
    shownames <- unique(d$show)
    sample_shows <- sample(shownames, length(shownames), replace=T)
    
    #create sample data frame with shows, repeated as necessary
    sample_d <- NULL
    for(show in sample_shows){
      sample_d <- rbind(sample_d, d[d$show == show,])
    }
    
    return(sample_d)
    
  }
  
  #bootstrap predictions from neg. bin. and frac. logit models
  get_pred <- function(d){
    
    #get a sample
    sample_d <- one_bs_sample(d)
    
    #get neg. bin. (counts) predictions
    fit_nb_sample <- glm.nb(Total ~ prepost*ideology + mins, data = sample_d)
    count <- predict(fit_nb_sample$fit, d_pred, type="response")
    
    #get frac. log. (proportions) predictions
    fit_fl_sample <- glm(prop_conc ~ prepost*ideology, sample_d,
                         family=quasibinomial('logit'))
    position <- predict(fit_fl_sample, d_pred, type="response")
    
    #make data frame
    pred_sample <- cbind(d_pred, count, position)
    
  }
  
  ##get bootstrapped predictions of mention counts and positions
  preds_boot <- rdply(1000, get_pred(d), .progress = "text")
  save("bootstrapped_predictions.Rdata", preds_boot)
  
  #get quantiles from bootstrapped predictions as CI bounds
  pred_nb_CI <- ddply(preds_boot, .(ideology, prepost), summarise,
                      CI_lo = quantile(count, .025), CI_hi = quantile(count, .975))
  pred_nb_CI <- merge(pred_nb, pred_nb_CI)
  pred_fl_CI <- ddply(preds_boot, .(ideology, prepost), summarise,
                      CI_lo = quantile(position, .025), CI_hi = quantile(position, .975))
  pred_fl_CI <- merge(pred_fl, pred_fl_CI)
  
  #write to file
  write.csv(pred_nb_CI, paste0("predictions_CIs/pred_count_", topic, ".csv"), row.names=F)
  write.csv(pred_fl_CI, paste0("predictions_CIs/pred_position_", topic, ".csv"), row.names=F)
  
}
