source("../radio_fragment_functions.R")
library(dplyr)
library(ggplot2)
library(ggpubr)

topic <- "clim" #clim for climate, gun for gun policy, immi for immigration


##Data prep

#get mention counts and event dates
mentions_tot <- read.csv(paste0("../data/mention_count_", topic, ".csv"))
mentions_tot$date <- as.Date(mentions_tot$date)
prepost <- read.csv("../data/event_prepost.csv")
prepost <- prepost[prepost$topic==topic,]

#get event week for each date
mentions_tot$week <- as.factor(mentions_tot$date)
levels(mentions_tot$week) <- sapply(levels(mentions_tot$week),
                                    event_week, prepost, topic)

#find peak related to each event (should happen somewhere in post-week)
peak <- as.data.frame(mentions_tot %>% group_by(week) %>% top_n(1, count))
peak <- peak[grep("post", peak$week),]
peak$date <- as.Date(peak$date)
peak$event <- sapply(strsplit(as.character(peak$week), "_"), "[[", 1)

#find out how many days after the start of the most recent event
#(max. 30 days ago)
peak <- peak[order(peak$date, decreasing = T),]
post_event <- function(date, df_peak){
  
  #go over each event (in reverse chronological order);
  #if date happened within 30 days of peak, measure distance
  #and stop going backwards
  event <- "none"
  days_after <- 99
  peak_size <- 0
  for(i in 1:nrow(df_peak)){
    
    after_peak <- date - df_peak$date[i]
    if(after_peak >= 0 & after_peak < 30) {
      event <- peak$event[i]
      days_after <- after_peak
      peak_size <- peak$count[i]
      break
    }
    
  }
  
  return(c(event_after=event,
           days_after=days_after,
           peak_size=peak_size))
  
}
after <- as.data.frame(t(sapply(mentions_tot$date, post_event, peak)))
after$days_after <- as.numeric(as.character(after$days_after))
after$peak_size <- as.numeric(as.character(after$peak_size))
mentions_tot <- cbind(mentions_tot, after)

#delete observations not after any event,
#and observations that overlap with post-week of the next event
mentions_tot$event_week <- sapply(strsplit(as.character(mentions_tot$week), "_"), "[[", 1)
mentions_tot$overlap <- (mentions_tot$event_week != mentions_tot$event_after) &
  grepl("post", mentions_tot$week)
print(sum(mentions_tot$overlap))
mentions_tot <- mentions_tot[mentions_tot$days_after != 99 & !mentions_tot$overlap ,]

#for gun control, leave out days coming after Thousand Oaks shooting
if(topic=="gun"){
  mentions_tot <- mentions_tot[!(mentions_tot$event_after == "PittsSyn" &
                                   mentions_tot$date >= as.Date("2018-11-07")),]
}

#add full event names
mentions_tot <- merge(mentions_tot, prepost[,c("event_abbr","term")], 
                      by.x = "event_after", by.y = "event_abbr", all.x=T)
mentions_tot$event_full <- gsub(" ", "\n", mentions_tot$term)
mentions_tot$event_full <- gsub("Santa\nFe", "Santa Fe", mentions_tot$event_full)
mentions_tot$event_full <- gsub("High\nSchool", "High School", mentions_tot$event_full)


##Model and plot (pct w/o event intercept)

#% of peak attention
mentions_tot$pct <- mentions_tot$count / mentions_tot$peak_size
mentions_tot$logpct <- log2(mentions_tot$pct)

#model
fit <- lm(logpct ~ days_after - 1, data=mentions_tot)
start <- list(beta = fit$coefficients["days_after"])
fit_nls <- nls(pct ~ 2^(beta*days_after),
             data = mentions_tot, start=start)
coef(fit_nls)
-coef(fit_nls)^(-1)

#trend prediction
prediction <- data.frame(days_after=0:30)
prediction$pct <- predict(fit_nls, prediction)
ggplot(prediction, aes(x=days_after, y=pct)) +
  geom_line() +
  geom_point(data=mentions_tot, aes(color=event_full)) +
  ylim(0, 1) +
  xlab("days after event") + ylab("mentions (% of peak)") + 
  labs(color = "event") +
  theme(legend.text = element_text(margin = margin(t = 10)))
ggsave(paste0("figures/longterm_", topic,  ".jpeg"))